@@ -11,6 +11,7 @@ from swh.model.model import (
     OriginVisitStatus,
     Sha1Git,
     Snapshot,
+    SnapshotBranch,
     TargetType,
 )
 from swh.storage.algos.origin import (
@@ -164,3 +165,48 @@ def visits_and_snapshots_get_from_revision(
                     and branch.target == revision_id
                 ):  # snapshot found
                     yield (visit, visit_status, snapshot)
+
+
+def snapshot_resolve_alias(
+    storage: StorageInterface, snapshot_id: Sha1Git, alias_name: bytes
+) -> Optional[SnapshotBranch]:
+    """
+    Transitively resolve snapshot branch alias to its real target, and return it;
+    ie. follows every branch that is an alias, until a branch that isn't an alias
+    is found.
+
+    Args:
+        storage: Storage instance
+        snapshot_id: snapshot identifier
+        alias_name: name of the branch alias to resolve
+
+    Returns:
+        The first branch that isn't an alias, in the alias chain; or None if
+        there is no such branch (ie. either because of a cycle alias, or a dangling
+        branch).
+    """
+    snapshot = storage.snapshot_get_branches(
+        snapshot_id, branches_from=alias_name, branches_count=1
+    )
+    if snapshot is None:
+        return None
+
+    if alias_name not in snapshot["branches"]:
+        return None
+
+    last_branch = snapshot["branches"][alias_name]
+    seen_aliases = {alias_name}
+
+    while last_branch is not None and last_branch.target_type == TargetType.ALIAS:
+        if last_branch.target in seen_aliases:
+            return None
+
+        alias_target = last_branch.target
+        snapshot = storage.snapshot_get_branches(
+            snapshot_id, branches_from=alias_target, branches_count=1
+        )
+        assert snapshot is not None
+        last_branch = snapshot["branches"].get(alias_target)
+        seen_aliases.add(alias_target)
+
+    return last_branch
