@@ -19,7 +19,6 @@ import logging
 from typing import Any, Callable, Dict
 
 from swh.core.db import BaseDb
-from swh.journal.writer.kafka import KafkaJournalWriter
 from swh.model.model import (
     BaseModel,
     Directory,
@@ -37,6 +36,7 @@ from swh.storage.postgresql.converters import (
     db_to_revision,
 )
 from swh.storage.replay import object_converter_fn
+from swh.storage.writer import JournalWriter
 
 logger = logging.getLogger(__name__)
 
@@ -46,7 +46,7 @@ PARTITION_KEY = {
     "directory": "id",
     "metadata_authority": "type, url",
     "metadata_fetcher": "name, version",
-    "raw_extrinsic_metadata": "id",
+    "raw_extrinsic_metadata": "target",
     "revision": "revision.id",
     "release": "release.id",
     "snapshot": "id",
@@ -80,7 +80,7 @@ COLUMNS = {
     "metadata_fetcher": ["name", "version", "metadata",],
     "raw_extrinsic_metadata": [
         "raw_extrinsic_metadata.type",
-        "raw_extrinsic_metadata.id",
+        "raw_extrinsic_metadata.target",
         "metadata_authority.type",
         "metadata_authority.url",
         "metadata_fetcher.name",
@@ -442,7 +442,7 @@ def fetch(db, obj_type, start, end):
             else:
                 record = object_converter_fn[obj_type](record)
 
-            logger.debug("record: %s" % record)
+            logger.debug("record: %s", record)
             yield record
 
 
@@ -453,7 +453,7 @@ def _format_range_bound(bound):
         return str(bound)
 
 
-MANDATORY_KEYS = ["brokers", "storage_dbconn", "prefix", "client_id"]
+MANDATORY_KEYS = ["storage", "journal_writer"]
 
 
 class JournalBackfiller:
@@ -480,6 +480,12 @@ class JournalBackfiller:
                 " provided: %s" % (",".join(missing_keys),)
             )
 
+        if "cls" not in config["storage"] or config["storage"]["cls"] != "local":
+            raise ValueError(
+                "swh storage backfiller must be configured to use a local"
+                " (PostgreSQL) storage"
+            )
+
     def parse_arguments(self, object_type, start_object, end_object):
         """Parse arguments
 
@@ -519,12 +525,10 @@ class JournalBackfiller:
             object_type, start_object, end_object
         )
 
-        db = BaseDb.connect(self.config["storage_dbconn"])
-        writer = KafkaJournalWriter(
-            brokers=self.config["brokers"],
-            prefix=self.config["prefix"],
-            client_id=self.config["client_id"],
-        )
+        db = BaseDb.connect(self.config["storage"]["db"])
+        writer = JournalWriter({"cls": "kafka", **self.config["journal_writer"]})
+        assert writer.journal is not None
+
         for range_start, range_end in RANGE_GENERATORS[object_type](
             start_object, end_object
         ):
@@ -535,12 +539,15 @@ class JournalBackfiller:
                 _format_range_bound(range_end),
             )
 
-            for obj in fetch(db, object_type, start=range_start, end=range_end,):
-                if dry_run:
-                    continue
-                writer.write_addition(object_type=object_type, object_=obj)
+            objects = fetch(db, object_type, start=range_start, end=range_end)
 
-            writer.producer.flush()
+            if not dry_run:
+                writer.write_additions(object_type, objects)
+            else:
+                # only consume the objects iterator to check for any potential
+                # decoding/encoding errors
+                for obj in objects:
+                    pass
 
 
 if __name__ == "__main__":
