@@ -29,7 +29,9 @@ import re
 import sys
 import time
 from typing import Any, Dict, Optional
+from urllib.error import HTTPError
 from urllib.parse import unquote, urlparse
+from urllib.request import urlopen
 
 import iso8601
 import psycopg2
@@ -56,13 +58,21 @@ ATOM_NS = "http://www.w3.org/2005/Atom"
 ATOM_KEYS = ["id", "author", "external_identifier", "title"]
 
 # columns of the revision table (of the storage DB)
-REVISION_COLS = ["id", "date", "committer_date", "type", "message", "metadata"]
+REVISION_COLS = [
+    "id",
+    "directory",
+    "date",
+    "committer_date",
+    "type",
+    "message",
+    "metadata",
+]
 
 # columns of the tables of the deposit DB
 DEPOSIT_COLS = [
     "deposit.id",
     "deposit.external_id",
-    "deposit.swh_id_context",
+    "deposit.swhid_context",
     "deposit.status",
     "deposit_request.metadata",
     "deposit_request.date",
@@ -108,22 +118,171 @@ AUTHORITIES = {
 # Regular expression for the format of revision messages written by the
 # deposit loader
 deposit_revision_message_re = re.compile(
-    b"(?P<client>[a-z]*): "
-    b"Deposit (?P<deposit_id>[0-9]+) in collection (?P<collection>[a-z]+).*"
+    b"(?P<client>[a-z-]*): "
+    b"Deposit (?P<deposit_id>[0-9]+) in collection (?P<collection>[a-z-]+).*"
 )
 
 
 # not reliable, because PyPI allows arbitrary names
 def pypi_project_from_filename(filename):
+    original_filename = filename
+    if filename.endswith(".egg"):
+        return None
+    elif filename == "mongomotor-0.13.0.n.tar.gz":
+        return "mongomotor"
+    elif re.match(r"datahaven-rev[0-9]+\.tar\.gz", filename):
+        return "datahaven"
+    elif re.match(r"Dtls-[0-9]\.[0-9]\.[0-9]\.sdist_with_openssl\..*", filename):
+        return "Dtls"
+    elif re.match(r"(gae)?pytz-20[0-9][0-9][a-z]\.(tar\.gz|zip)", filename):
+        return filename.split("-", 1)[0]
+    elif filename.startswith(("powny-", "obedient.powny-",)):
+        return filename.split("-")[0]
+    elif filename.startswith("devpi-theme-16-"):
+        return "devpi-theme-16"
+    elif re.match("[^-]+-[0-9]+.tar.gz", filename):
+        return filename.split("-")[0]
+    elif filename == "ohai-1!0.tar.gz":
+        return "ohai"
+    elif filename == "collective.topicitemsevent-0.1dvl.tar.gz":
+        return "collective.topicitemsevent"
+    elif filename.startswith(
+        ("SpiNNStorageHandlers-1!", "sPyNNakerExternalDevicesPlugin-1!")
+    ):
+        return filename.split("-")[0]
+    elif filename.startswith("limnoria-201"):
+        return "limnoria"
+    elif filename.startswith("pytz-20"):
+        return "pytz"
+    elif filename.startswith("youtube_dl_server-alpha."):
+        return "youtube_dl_server"
+    elif filename == "json-extensions-b76bc7d.tar.gz":
+        return "json-extensions"
+    elif filename == "LitReview-0.6989ev.tar.gz":
+        # typo of "dev"
+        return "LitReview"
+    elif filename.startswith("django_options-r"):
+        return "django_options"
+    elif filename == "Greater than, equal, or less Library-0.1.tar.gz":
+        return "Greater-than-equal-or-less-Library"
+    elif filename.startswith("upstart--main-"):
+        return "upstart"
+    elif filename == "duckduckpy0.1.tar.gz":
+        return "duckduckpy"
+    elif filename == "QUI for MPlayer snapshot_9-14-2011.zip":
+        return "QUI-for-MPlayer"
+    elif filename == "Eddy's Memory Game-1.0.zip":
+        return "Eddy-s-Memory-Game"
+    elif filename == "jekyll2nikola-0-0-1.tar.gz":
+        return "jekyll2nikola"
+    elif filename.startswith("ore.workflowed"):
+        return "ore.workflowed"
+    elif re.match("instancemanager-[0-9]*", filename):
+        return "instancemanager"
+    elif filename == "OrzMC_W&L-1.0.0.tar.gz":
+        return "OrzMC-W-L"
+    elif filename == "use0mk.tar.gz":
+        return "use0mk"
+    elif filename == "play-0-develop-1-gd67cd85.tar.gz":
+        return "play"
+    filename = filename.replace(" ", "-")
+
     match = re.match(
-        r"^(?P<project_name>[a-zA-Z0-9_.-]+)"
-        r"-[0-9.]+([a-z]+[0-9]+)?(dev|\.dev[0-9]+)?(-[a-z][a-z0-9]*)?\.(tar\.gz|zip)$",
+        r"^(?P<project_name>[a-z_.-]+)"  # project name
+        r"\.(tar\.gz|tar\.bz2|tgz|zip)$",  # extension
+        filename,
+        re.I,
+    )
+    if match:
+        return match.group("project_name")
+
+    # First try with a rather strict format, but that allows accidentally
+    # matching the version as part of the package name
+    match = re.match(
+        r"^(?P<project_name>[a-z0-9_.]+?([-_][a-z][a-z0-9.]+?)*?)"  # project name
+        r"-v?"
+        r"([0-9]+!)?"  # epoch
+        r"[0-9_.]+([a-z]+[0-9]+)?"  # "main" version
+        r"([.-]?(alpha|beta|dev|post|pre|rc)(\.?[0-9]+)?)*"  # development status
+        r"([.-]?20[012][0-9]{5,9})?"  # date
+        r"([.-]g?[0-9a-f]+)?"  # git commit
+        r"([-+]py(thon)?(3k|[23](\.?[0-9]{1,2})?))?"  # python version
+        r"\.(tar\.gz|tar\.bz2|tgz|zip)$",  # extension
+        filename,
+        re.I,
+    )
+    if match:
+        return match.group("project_name")
+
+    # If that doesn't work, give up on trying to parse version suffixes,
+    # and just find the first version-like occurrence in the file name
+
+    match = re.match(
+        r"^(?P<project_name>[a-z0-9_.-]+?)"  # project name
+        r"[-_.]v?"
+        r"([0-9]+!)?"  # epoch
+        r"("  # "main" version
+        r"[0-9_]+\.[0-9_.]+([a-z]+[0-9]+)?"  # classic version number
+        r"|20[012][0-9]{5,9}"  # date as integer
+        r"|20[012][0-9]-[01][0-9]-[0-3][0-9]"  # date as ISO 8601
+        r")"  # end of "main" version
+        r"[a-z]?(dev|pre)?"  # direct version suffix
+        r"([._-].*)?"  # extra suffixes
+        r"\.(tar\.gz|tar\.bz2|tgz|zip)$",  # extension
+        filename,
+        re.I,
+    )
+    if match:
+        return match.group("project_name")
+
+    # If that still doesn't work, give one last chance if there's only one
+    # dash or underscore in the name
+
+    match = re.match(
+        r"^(?P<project_name>[^_-]+)"  # project name
+        r"[_-][^_-]+"  # version
+        r"\.(tar\.gz|tar\.bz2|tgz|zip)$",  # extension
         filename,
     )
-    assert match, filename
+    assert match, original_filename
     return match.group("project_name")
 
 
+def pypi_origin_from_project_name(project_name: str) -> str:
+    return f"https://pypi.org/project/{project_name}/"
+
+
+def pypi_origin_from_filename(storage, rev_id: bytes, filename: str) -> Optional[str]:
+    project_name = pypi_project_from_filename(filename)
+    origin = pypi_origin_from_project_name(project_name)
+    # But unfortunately, the filename is user-provided, and doesn't
+    # necessarily match the package name on pypi. Therefore, we need
+    # to check it.
+    if _check_revision_in_origin(storage, origin, rev_id):
+        return origin
+
+    # if the origin we guessed does not exist, query the PyPI API with the
+    # project name we guessed. If only the capitalisation and dash/underscores
+    # are wrong (by far the most common case), PyPI kindly corrects them.
+    try:
+        resp = urlopen(f"https://pypi.org/pypi/{project_name}/json/")
+    except HTTPError as e:
+        assert e.code == 404
+        # nope; PyPI couldn't correct the wrong project name
+        return None
+    assert resp.code == 200, resp.code
+    project_name = json.load(resp)["info"]["name"]
+    origin = pypi_origin_from_project_name(project_name)
+
+    if _check_revision_in_origin(storage, origin, rev_id):
+        return origin
+    else:
+        # The origin exists, but the revision does not belong in it.
+        # This happens sometimes, as the filename we guessed the origin
+        # from is user-provided.
+        return None
+
+
 def cran_package_from_url(filename):
     match = re.match(
         r"^https://cran\.r-project\.org/src/contrib/"
@@ -228,15 +387,23 @@ _origins = set()
 
 
 def assert_origin_exists(storage, origin):
-    assert (
-        hashlib.sha1(origin.encode()).digest() in _origins  # very fast
-        or storage.origin_get([origin])[0] is not None  # slow, but up to date
-    ), origin
+    assert check_origin_exists(storage, origin), origin
+
+
+def check_origin_exists(storage, origin):
+    return (
+        (
+            hashlib.sha1(origin.encode()).digest() in _origins  # very fast
+            or storage.origin_get([origin])[0] is not None  # slow, but up to date
+        ),
+        origin,
+    )
 
 
 def load_metadata(
     storage,
     revision_id,
+    directory_id,
     discovery_date: datetime.datetime,
     metadata: Dict[str, Any],
     format: str,
@@ -245,16 +412,20 @@ def load_metadata(
     dry_run: bool,
 ):
     """Does the actual loading to swh-storage."""
+    directory_swhid = SWHID(
+        object_type="directory", object_id=hash_to_hex(directory_id)
+    )
     revision_swhid = SWHID(object_type="revision", object_id=hash_to_hex(revision_id))
     obj = RawExtrinsicMetadata(
-        type=MetadataTargetType.REVISION,
-        id=revision_swhid,
+        type=MetadataTargetType.DIRECTORY,
+        target=directory_swhid,
         discovery_date=discovery_date,
         authority=authority,
         fetcher=FETCHER,
         format=format,
         metadata=json.dumps(metadata).encode(),
         origin=origin,
+        revision=revision_swhid,
     )
     if not dry_run:
         storage.raw_extrinsic_metadata_add([obj])
@@ -310,9 +481,9 @@ def handle_deposit_row(
         date = deposit_request["deposit_request.date"]
         dates.add(date)
 
-        assert deposit_request["deposit.swh_id_context"], deposit_request
+        assert deposit_request["deposit.swhid_context"], deposit_request
         external_identifiers.add(deposit_request["deposit.external_id"])
-        swhids.add(deposit_request["deposit.swh_id_context"])
+        swhids.add(deposit_request["deposit.swhid_context"])
 
         # Client of the deposit
         provider_urls.add(deposit_request["deposit_client.provider_url"])
@@ -324,13 +495,23 @@ def handle_deposit_row(
                 assert metadata["@xmlns"] == ATOM_NS
                 assert metadata["@xmlns:codemeta"] in (CODEMETA_NS, [CODEMETA_NS])
                 format = NEW_DEPOSIT_FORMAT
-            else:
-                assert "{http://www.w3.org/2005/Atom}id" in metadata
+            elif "{http://www.w3.org/2005/Atom}id" in metadata:
                 assert (
                     "{https://doi.org/10.5063/SCHEMA/CODEMETA-2.0}author" in metadata
                     or "{http://www.w3.org/2005/Atom}author" in metadata
                 )
                 format = OLD_DEPOSIT_FORMAT
+            else:
+                # new format introduced in
+                # https://forge.softwareheritage.org/D4065
+                # it's the same as the first case, but with the @xmlns
+                # declarations stripped
+                # Most of them should have the "id", but some revisions,
+                # like 4d3890004fade1f4ec3bf7004a4af0c490605128, are missing
+                # this field
+                assert "id" in metadata or "title" in metadata
+                assert "codemeta:author" in metadata
+                format = NEW_DEPOSIT_FORMAT
             metadata_entries.append((date, format, metadata))
 
     if discovery_date is None:
@@ -418,6 +599,7 @@ def handle_deposit_row(
         load_metadata(
             storage,
             row["id"],
+            row["directory"],
             date,
             metadata,
             format,
@@ -478,6 +660,7 @@ def handle_row(row: Dict[str, Any], storage, deposit_cur, dry_run: bool):
                 load_metadata(
                     storage,
                     row["id"],
+                    row["directory"],
                     discovery_date,
                     metadata["extrinsic"]["raw"],
                     NPM_FORMAT,
@@ -501,6 +684,7 @@ def handle_row(row: Dict[str, Any], storage, deposit_cur, dry_run: bool):
                 load_metadata(
                     storage,
                     row["id"],
+                    row["directory"],
                     discovery_date,
                     metadata["extrinsic"]["raw"],
                     PYPI_FORMAT,
@@ -519,24 +703,33 @@ def handle_row(row: Dict[str, Any], storage, deposit_cur, dry_run: bool):
                 else:
                     package_name = cran_package_from_url(provider)
                     origin = f"https://cran.r-project.org/package={package_name}"
-                # TODO https://forge.softwareheritage.org/T2536
                 assert origin is not None
+
+                # Ideally we should assert the origin exists, but we can't:
+                # https://forge.softwareheritage.org/T2536
                 if (
                     hashlib.sha1(origin.encode()).digest() not in _origins
                     and storage.origin_get([origin])[0] is None
                 ):
-                    print("MISSING CRAN ORIGIN", hash_to_hex(row["id"]), origin)
                     return
 
                 raw_extrinsic_metadata = metadata["extrinsic"]["raw"]
 
                 # this is actually intrinsic, ignore it
-                del raw_extrinsic_metadata["version"]
+                if "version" in raw_extrinsic_metadata:
+                    del raw_extrinsic_metadata["version"]
 
                 # Copy the URL to the original_artifacts metadata
                 assert len(metadata["original_artifact"]) == 1
-                assert "url" not in metadata["original_artifact"][0]
-                metadata["original_artifact"][0]["url"] = raw_extrinsic_metadata["url"]
+                if "url" in metadata["original_artifact"][0]:
+                    assert (
+                        metadata["original_artifact"][0]["url"]
+                        == raw_extrinsic_metadata["url"]
+                    ), row
+                else:
+                    metadata["original_artifact"][0]["url"] = raw_extrinsic_metadata[
+                        "url"
+                    ]
                 del raw_extrinsic_metadata["url"]
 
                 assert (
@@ -545,7 +738,10 @@ def handle_row(row: Dict[str, Any], storage, deposit_cur, dry_run: bool):
 
                 del metadata["extrinsic"]
 
-            elif provider.startswith("https://nix-community.github.io/nixpkgs-swh/"):
+            elif (
+                provider.startswith("https://nix-community.github.io/nixpkgs-swh/")
+                or provider == "https://guix.gnu.org/sources.json"
+            ):
                 # nixguix loader
                 origin = provider
                 assert_origin_exists(storage, origin)
@@ -558,6 +754,7 @@ def handle_row(row: Dict[str, Any], storage, deposit_cur, dry_run: bool):
                 load_metadata(
                     storage,
                     row["id"],
+                    row["directory"],
                     discovery_date,
                     metadata["extrinsic"]["raw"],
                     NIXGUIX_FORMAT,
@@ -612,18 +809,31 @@ def handle_row(row: Dict[str, Any], storage, deposit_cur, dry_run: bool):
                     actual_metadata = metadata["extrinsic"]["raw"]["origin_metadata"][
                         "metadata"
                     ]
+                    if isinstance(actual_metadata, str):
+                        # new format introduced in
+                        # https://forge.softwareheritage.org/D4105
+                        actual_metadata = json.loads(actual_metadata)
                     if "@xmlns" in actual_metadata:
                         assert actual_metadata["@xmlns"] == ATOM_NS
                         assert actual_metadata["@xmlns:codemeta"] in (
                             CODEMETA_NS,
                             [CODEMETA_NS],
                         )
-                    else:
-                        assert "{http://www.w3.org/2005/Atom}id" in actual_metadata
+                    elif "{http://www.w3.org/2005/Atom}id" in actual_metadata:
                         assert (
                             "{https://doi.org/10.5063/SCHEMA/CODEMETA-2.0}author"
                             in actual_metadata
                         )
+                    else:
+                        # new format introduced in
+                        # https://forge.softwareheritage.org/D4065
+                        # it's the same as the first case, but with the @xmlns
+                        # declarations stripped
+                        # Most of them should have the "id", but some revisions,
+                        # like 4d3890004fade1f4ec3bf7004a4af0c490605128, are missing
+                        # this field
+                        assert "id" in actual_metadata or "title" in actual_metadata
+                        assert "codemeta:author" in actual_metadata
 
                     (origin, discovery_date) = handle_deposit_row(
                         row, discovery_date, origin, storage, deposit_cur, dry_run
@@ -649,6 +859,7 @@ def handle_row(row: Dict[str, Any], storage, deposit_cur, dry_run: bool):
             load_metadata(
                 storage,
                 row["id"],
+                row["directory"],
                 discovery_date,
                 metadata["package"],
                 NPM_FORMAT,
@@ -756,26 +967,16 @@ def handle_row(row: Dict[str, Any], storage, deposit_cur, dry_run: bool):
 
             assert len(metadata["original_artifact"]) == 1
 
-            project_name = pypi_project_from_filename(
-                metadata["original_artifact"][0]["filename"]
+            origin = pypi_origin_from_filename(
+                storage, row["id"], metadata["original_artifact"][0]["filename"]
             )
-            origin = f"https://pypi.org/project/{project_name}/"
-            # But unfortunately, the filename is user-provided, and doesn't
-            # necessarily match the package name on pypi. Therefore, we need
-            # to check it.
-            if not _check_revision_in_origin(storage, origin, row["id"]):
-                origin = None
 
             if "project" in metadata:
                 # pypi loader format 2
-
-                # same reason as above, we can't do this:
-                #   if metadata["project"]:
-                #       assert metadata["project"]["name"] == project_name
-
                 load_metadata(
                     storage,
                     row["id"],
+                    row["directory"],
                     discovery_date,
                     metadata["project"],
                     PYPI_FORMAT,
@@ -871,6 +1072,7 @@ def handle_row(row: Dict[str, Any], storage, deposit_cur, dry_run: bool):
             load_metadata(
                 storage,
                 row["id"],
+                row["directory"],
                 discovery_date,
                 metadata["original_artifact"],
                 ORIGINAL_ARTIFACT_FORMAT,
@@ -933,7 +1135,17 @@ def iter_revision_rows(storage_dbconn: str, first_id: Sha1Git):
 def main(storage_dbconn, storage_url, deposit_dbconn, first_id, dry_run):
     storage_db = BaseDb.connect(storage_dbconn)
     deposit_db = BaseDb.connect(deposit_dbconn)
-    storage = get_storage("remote", url=storage_url)
+    storage = get_storage(
+        "pipeline",
+        steps=[
+            {"cls": "retry"},
+            {
+                "cls": "local",
+                "db": storage_dbconn,
+                "objstorage": {"cls": "memory", "args": {}},
+            },
+        ],
+    )
 
     if not dry_run:
         create_fetchers(storage_db)
